@@ -13,15 +13,14 @@ KRB Feb 2017
 from landlab.core.utils import as_id_array
 from landlab.utils.return_array import return_array_at_node
 from landlab import BAD_INDEX_VALUE, CLOSED_BOUNDARY
+
 UNDEFINED_INDEX = BAD_INDEX_VALUE
 from landlab import VoronoiDelaunayGrid  # for type tests
 
 import numpy as np
 
 
-def flow_directions_dinf(grid,
-                         elevs='topographic__elevation',
-                         baselevel_nodes=None):
+def flow_directions_dinf(grid, elevs="topographic__elevation", baselevel_nodes=None):
     """
     Find Dinfinity flow directions and proportions on a raster grid.
 
@@ -53,6 +52,10 @@ def flow_directions_dinf(grid,
         For each receiver, the proportion of flow (between 0 and 1) is given.
         A proportion of zero indicates that the link does not have flow along
         it.
+    slopes: ndarray of size (num nodes, max neighbors at node)
+        For each node in the array ``recievers``, the slope value (positive
+        downhill) in the direction of flow. If no flow occurs (value of
+        ``recievers`` is -1), then this array is set to 0.
     steepest_slope : ndarray
         The slope value (positive downhill) in the direction of flow.
     steepest_receiver : ndarray
@@ -80,7 +83,7 @@ def flow_directions_dinf(grid,
     >>> _ = grid.add_field('topographic__elevation',
     ...                     2.*grid.node_x+grid.node_y,
     ...                     at = 'node')
-    >>> (receivers, proportions,
+    >>> (receivers, proportions, slopes,
     ... steepest_slope, steepest_receiver,
     ... sink, receiver_links, steepest_link) = flow_directions_dinf(grid)
     >>> receivers
@@ -108,7 +111,7 @@ def flow_directions_dinf(grid,
     the (implied) field name 'topographic__elevation'.
 
     >>> z = grid['node']['topographic__elevation']
-    >>> (receivers, proportions,
+    >>> (receivers, proportions, slopes,
     ... steepest_slope, steepest_receiver,
     ... sink, receiver_links, steepest_link) = flow_directions_dinf(grid, z)
     >>> receivers
@@ -121,7 +124,16 @@ def flow_directions_dinf(grid,
            [ 3,  4],
            [ 6,  3],
            [ 7,  4]])
-
+    >>> slopes
+    array([[-2.        , -0.        ],
+           [ 2.        ,  0.70710678],
+           [ 2.        ,  0.70710678],
+           [ 1.        , -0.70710678],
+           [ 2.        ,  2.12132034],
+           [ 2.        ,  2.12132034],
+           [ 1.        , -0.70710678],
+           [ 2.        ,  2.12132034],
+           [ 2.        ,  2.12132034]])
     >>> proportions
     array([[ 1.        ,  0.        ],
            [ 1.        , -0.        ],
@@ -135,8 +147,9 @@ def flow_directions_dinf(grid,
     """
     # grid type testing
     if isinstance(grid, VoronoiDelaunayGrid):
-        raise NotImplementedError('Dinfinity is currently implemented for'
-                                  ' Raster grids only')
+        raise NotImplementedError(
+            "Dinfinity is currently implemented for" " Raster grids only"
+        )
     # get elevs
     elevs = return_array_at_node(grid, elevs)
 
@@ -167,77 +180,90 @@ def flow_directions_dinf(grid,
     n_at_node = grid.adjacent_nodes_at_node
     dn_at_node = grid.diagonal_adjacent_nodes_at_node
 
-    triangle_neighbors_at_node = np.stack([np.vstack((n_at_node[:,0], dn_at_node[:,0])),
-                                           np.vstack((n_at_node[:,1], dn_at_node[:,0])),
-                                           np.vstack((n_at_node[:,1], dn_at_node[:,1])),
-                                           np.vstack((n_at_node[:,2], dn_at_node[:,1])),
-                                           np.vstack((n_at_node[:,2], dn_at_node[:,2])),
-                                           np.vstack((n_at_node[:,3], dn_at_node[:,2])),
-                                           np.vstack((n_at_node[:,3], dn_at_node[:,3])),
-                                           np.vstack((n_at_node[:,0], dn_at_node[:,3]))],
-                                          axis=-1)
-    triangle_neighbors_at_node = triangle_neighbors_at_node.swapaxes(0,1)
+    triangle_neighbors_at_node = np.stack(
+        [
+            np.vstack((n_at_node[:, 0], dn_at_node[:, 0])),
+            np.vstack((n_at_node[:, 1], dn_at_node[:, 0])),
+            np.vstack((n_at_node[:, 1], dn_at_node[:, 1])),
+            np.vstack((n_at_node[:, 2], dn_at_node[:, 1])),
+            np.vstack((n_at_node[:, 2], dn_at_node[:, 2])),
+            np.vstack((n_at_node[:, 3], dn_at_node[:, 2])),
+            np.vstack((n_at_node[:, 3], dn_at_node[:, 3])),
+            np.vstack((n_at_node[:, 0], dn_at_node[:, 3])),
+        ],
+        axis=-1,
+    )
+    triangle_neighbors_at_node = triangle_neighbors_at_node.swapaxes(0, 1)
 
     # next create, triangle links at node
     l_at_node = grid.d8s_at_node[:, :4]
     dl_at_node = grid.d8s_at_node[:, 4:]
-    triangle_links_at_node = np.stack([np.vstack((l_at_node[:,0], dl_at_node[:,0])),
-                                       np.vstack((l_at_node[:,1], dl_at_node[:,0])),
-                                       np.vstack((l_at_node[:,1], dl_at_node[:,1])),
-                                       np.vstack((l_at_node[:,2], dl_at_node[:,1])),
-                                       np.vstack((l_at_node[:,2], dl_at_node[:,2])),
-                                       np.vstack((l_at_node[:,3], dl_at_node[:,2])),
-                                       np.vstack((l_at_node[:,3], dl_at_node[:,3])),
-                                       np.vstack((l_at_node[:,0], dl_at_node[:,3]))],
-                                      axis=-1)
-    triangle_links_at_node = triangle_links_at_node.swapaxes(0,1)
+    triangle_links_at_node = np.stack(
+        [
+            np.vstack((l_at_node[:, 0], dl_at_node[:, 0])),
+            np.vstack((l_at_node[:, 1], dl_at_node[:, 0])),
+            np.vstack((l_at_node[:, 1], dl_at_node[:, 1])),
+            np.vstack((l_at_node[:, 2], dl_at_node[:, 1])),
+            np.vstack((l_at_node[:, 2], dl_at_node[:, 2])),
+            np.vstack((l_at_node[:, 3], dl_at_node[:, 2])),
+            np.vstack((l_at_node[:, 3], dl_at_node[:, 3])),
+            np.vstack((l_at_node[:, 0], dl_at_node[:, 3])),
+        ],
+        axis=-1,
+    )
+    triangle_links_at_node = triangle_links_at_node.swapaxes(0, 1)
 
     # next create link directions and active link directions at node
     # link directions
     ld_at_node = grid.link_dirs_at_node
     dld_at_node = grid.diagonal_dirs_at_node
-    triangle_link_dirs_at_node = np.stack([np.vstack((ld_at_node[:,0], dld_at_node[:,0])),
-                                           np.vstack((ld_at_node[:,1], dld_at_node[:,0])),
-                                           np.vstack((ld_at_node[:,1], dld_at_node[:,1])),
-                                           np.vstack((ld_at_node[:,2], dld_at_node[:,1])),
-                                           np.vstack((ld_at_node[:,2], dld_at_node[:,2])),
-                                           np.vstack((ld_at_node[:,3], dld_at_node[:,2])),
-                                           np.vstack((ld_at_node[:,3], dld_at_node[:,3])),
-                                           np.vstack((ld_at_node[:,0], dld_at_node[:,3]))],
-                                          axis=-1)
-    triangle_link_dirs_at_node = triangle_link_dirs_at_node.swapaxes(0,1)
+    triangle_link_dirs_at_node = np.stack(
+        [
+            np.vstack((ld_at_node[:, 0], dld_at_node[:, 0])),
+            np.vstack((ld_at_node[:, 1], dld_at_node[:, 0])),
+            np.vstack((ld_at_node[:, 1], dld_at_node[:, 1])),
+            np.vstack((ld_at_node[:, 2], dld_at_node[:, 1])),
+            np.vstack((ld_at_node[:, 2], dld_at_node[:, 2])),
+            np.vstack((ld_at_node[:, 3], dld_at_node[:, 2])),
+            np.vstack((ld_at_node[:, 3], dld_at_node[:, 3])),
+            np.vstack((ld_at_node[:, 0], dld_at_node[:, 3])),
+        ],
+        axis=-1,
+    )
+    triangle_link_dirs_at_node = triangle_link_dirs_at_node.swapaxes(0, 1)
 
-#    # active link directions.
-#    ald_at_node = grid.active_link_dirs_at_node
-#    adld_at_node = grid.active_diagonal_dirs_at_node
-#
-#    triangle_active_link_dirs_at_node = np.stack([np.vstack((ald_at_node[:,0], adld_at_node[:,0])),
-#                                                  np.vstack((ald_at_node[:,1], adld_at_node[:,0])),
-#                                                  np.vstack((ald_at_node[:,1], adld_at_node[:,1])),
-#                                                  np.vstack((ald_at_node[:,2], adld_at_node[:,1])),
-#                                                  np.vstack((ald_at_node[:,2], adld_at_node[:,2])),
-#                                                  np.vstack((ald_at_node[:,3], adld_at_node[:,2])),
-#                                                  np.vstack((ald_at_node[:,3], adld_at_node[:,3])),
-#                                                  np.vstack((ald_at_node[:,0], adld_at_node[:,3]))],
-#                                                 axis=-1)
-#    triangle_active_link_dirs_at_node = triangle_active_link_dirs_at_node.swapaxes(0,1)
-#
+    #    # active link directions.
+    #    ald_at_node = grid.active_link_dirs_at_node
+    #    adld_at_node = grid.active_diagonal_dirs_at_node
+    #
+    #    triangle_active_link_dirs_at_node = np.stack([np.vstack((ald_at_node[:,0], adld_at_node[:,0])),
+    #                                                  np.vstack((ald_at_node[:,1], adld_at_node[:,0])),
+    #                                                  np.vstack((ald_at_node[:,1], adld_at_node[:,1])),
+    #                                                  np.vstack((ald_at_node[:,2], adld_at_node[:,1])),
+    #                                                  np.vstack((ald_at_node[:,2], adld_at_node[:,2])),
+    #                                                  np.vstack((ald_at_node[:,3], adld_at_node[:,2])),
+    #                                                  np.vstack((ald_at_node[:,3], adld_at_node[:,3])),
+    #                                                  np.vstack((ald_at_node[:,0], adld_at_node[:,3]))],
+    #                                                 axis=-1)
+    #    triangle_active_link_dirs_at_node = triangle_active_link_dirs_at_node.swapaxes(0,1)
+    #
     # need to create a list of diagonal links since it doesn't exist.
     diag_links = np.sort(np.unique(grid.d8s_at_node[:, 4:]))
-    diag_links = diag_links[diag_links>0]
+    diag_links = diag_links[diag_links > 0]
 
     # calculate graidents across diagonals and orthogonals
     diag_grads = grid._calculate_gradients_at_d8_links(elevs)
     ortho_grads = grid.calc_grad_at_link(elevs)
 
     # finally compile link slopes
-    link_slope = np.hstack((ortho_grads,
-                            diag_grads))
+    link_slope = np.hstack((ortho_grads, diag_grads))
 
     # Construct the array of slope to triangles at node. This also will adjust
     # for the slope convention based on the direction of the links.
     # this is a (nnodes, 2, 8) array
-    slopes_to_triangles_at_node = link_slope[triangle_links_at_node]*triangle_link_dirs_at_node
+    slopes_to_triangles_at_node = (
+        link_slope[triangle_links_at_node] * triangle_link_dirs_at_node
+    )
 
     # identify where nodes are closed.
     closed_triangle_neighbors = closed_nodes[triangle_neighbors_at_node]
@@ -251,13 +277,17 @@ def flow_directions_dinf(grid,
 
     # construct d1 and d2, we know these because we know where the orthogonal
     # links are
-    diag_length = ((grid.dx)**2+(grid.dy)**2)**0.5
+    diag_length = ((grid.dx) ** 2 + (grid.dy) ** 2) ** 0.5
 
     # for irregular grids, d1 and d2 will need to be matricies
-    d1 = np.array([grid.dx, grid.dy, grid.dy, grid.dx, grid.dx, grid.dy, grid.dy, grid.dy])
-    d2 = np.array([grid.dx, grid.dx, grid.dy, grid.dy, grid.dx, grid.dx, grid.dy, grid.dy])
+    d1 = np.array(
+        [grid.dx, grid.dy, grid.dy, grid.dx, grid.dx, grid.dy, grid.dy, grid.dy]
+    )
+    d2 = np.array(
+        [grid.dx, grid.dx, grid.dy, grid.dy, grid.dx, grid.dx, grid.dy, grid.dy]
+    )
 
-    thresh = np.arctan(d2/d1)
+    thresh = np.arctan(d2 / d1)
 
     ##### Step 4, Initialize receiver and proportion arrays
     receivers = UNDEFINED_INDEX * np.ones((num_nodes, num_receivers), dtype=int)
@@ -276,13 +306,13 @@ def flow_directions_dinf(grid,
     e0 = elevs[node_id]
 
     # e1 is the point on the orthogoanal edges
-    e1 = elevs[triangle_neighbors_at_node[:,0,:]]
+    e1 = elevs[triangle_neighbors_at_node[:, 0, :]]
     # e2 is the point on the diagonal edges
-    e2 = elevs[triangle_neighbors_at_node[:,1,:]]
+    e2 = elevs[triangle_neighbors_at_node[:, 1, :]]
 
     # mask out where nodes do not exits (e.g. triangle_neighbors_at_node == -1)
-    e2[triangle_neighbors_at_node[:,1,:] == -1] = np.nan
-    e1[triangle_neighbors_at_node[:,0,:] == -1] = np.nan
+    e2[triangle_neighbors_at_node[:, 1, :] == -1] = np.nan
+    e1[triangle_neighbors_at_node[:, 0, :] == -1] = np.nan
 
     # loop through and calculate s1 and s2
     # this will only loop nfacets times.
@@ -290,12 +320,12 @@ def flow_directions_dinf(grid,
     s2 = np.empty_like(e2)
 
     for i in range(num_facets):
-        s1[:,i] = (e0 - e1[:, i])/d1[i]
-        s2[:,i] = (e1[:, i]- e2[:, i])/d2[i]
+        s1[:, i] = (e0 - e1[:, i]) / d1[i]
+        s2[:, i] = (e1[:, i] - e2[:, i]) / d2[i]
 
     # calculate r and s, the direction and magnitude
     r = np.arctan2(s2, s1)
-    s = ((s1**2)+(s2**2))**0.5
+    s = ((s1 ** 2) + (s2 ** 2)) ** 0.5
 
     r[np.isnan(r)] = 0
     # adjust r if it doesn't sit in the realm of (0, arctan(d2,d1))
@@ -308,12 +338,12 @@ def flow_directions_dinf(grid,
     for i in range(num_facets):
         too_big = r[:, i] > thresh[i]
         radj[too_big, i] = thresh[i]
-        s[too_big, i] = (e0[too_big] - e2[too_big, i])/diag_length
+        s[too_big, i] = (e0[too_big] - e2[too_big, i]) / diag_length
 
     # calculate the geospatial version of r based on radj
     rg = np.empty_like(r)
     for i in range(num_facets):
-        rg[:, i] = (af[i]*radj[:, i]) + (ac[i]*np.pi/2.)
+        rg[:, i] = (af[i] * radj[:, i]) + (ac[i] * np.pi / 2.)
 
     # set slopes that are nan to zero
     s[np.isnan(s)] = 0
@@ -325,8 +355,8 @@ def flow_directions_dinf(grid,
     steepest_triangle = tri_numbers[steepest_sort[:, -1]]
 
     # initialize arrays for the steepest rg and steepest s
-    steepest_rg = np.empty_like(node_id, dtype = float)
-    steepest_s = np.empty_like(node_id, dtype = float)
+    steepest_rg = np.empty_like(node_id, dtype=float)
+    steepest_s = np.empty_like(node_id, dtype=float)
 
     for n in node_id:
         steepest_rg[n] = rg[n, steepest_sort[n, -1]]
@@ -334,19 +364,21 @@ def flow_directions_dinf(grid,
         steepest_s[n] = s[n, steepest_sort[n, -1]]
         receivers[n, :] = triangle_neighbors_at_node[n, :, steepest_sort[n, -1]]
         receiver_links[n, :] = triangle_links_at_node[n, :, steepest_sort[n, -1]]
-        slopes_to_receivers[n, :] = slopes_to_triangles_at_node[n, :, steepest_sort[n, -1]]
+        slopes_to_receivers[n, :] = slopes_to_triangles_at_node[
+            n, :, steepest_sort[n, -1]
+        ]
 
     # construct the baseline for proportions
-    rg_baseline = np.array([0., 1., 1., 2., 2., 3., 3., 4])*np.pi/2.
-    #rg_baseline = np.array([0., 0.5, 1., 1.5, 2., 2.5, 3., 3.5])*np.pi/4.
+    rg_baseline = np.array([0., 1., 1., 2., 2., 3., 3., 4]) * np.pi / 2.
+    # rg_baseline = np.array([0., 0.5, 1., 1.5, 2., 2.5, 3., 3.5])*np.pi/4.
 
     # calculate alpha1 and alpha 2
-    alpha2 = (steepest_rg-rg_baseline[steepest_triangle])*af[steepest_triangle]
+    alpha2 = (steepest_rg - rg_baseline[steepest_triangle]) * af[steepest_triangle]
     alpha1 = thresh[steepest_triangle] - alpha2
 
     # calculate proportions from alpha
-    proportions[:, 0] = (alpha1)/(alpha1+alpha2)
-    proportions[:, 1] = (alpha2)/(alpha1+alpha2)
+    proportions[:, 0] = (alpha1) / (alpha1 + alpha2)
+    proportions[:, 1] = (alpha2) / (alpha1 + alpha2)
 
     ### END OF THE Tarboton algorithm, start of work to make this code mesh
     # with other landlab flow directing algorithms.
@@ -367,23 +399,21 @@ def flow_directions_dinf(grid,
     # if drains to one closed receiver, check that the open receiver actually
     # gets flow. If so, route all to the open receiver. If the receiver getting
     # all the flow is closed, then drain to self.
-    all_flow_to_closed = np.sum(receiver_closed*proportions, axis=1) == 1
+    all_flow_to_closed = np.sum(receiver_closed * proportions, axis=1) == 1
     drains_to_self[all_flow_to_closed] = True
 
-    drains_to_one_closed = receiver_closed.sum(axis=1)==1
+    drains_to_one_closed = receiver_closed.sum(axis=1) == 1
     fix_flow = drains_to_one_closed * (all_flow_to_closed == False)
-    first_column_has_closed = np.array(receiver_closed[:, 0]*fix_flow,
-                                       dtype=bool)
-    second_column_has_closed = np.array(receiver_closed[:, 1]*fix_flow,
-                                        dtype=bool)
+    first_column_has_closed = np.array(receiver_closed[:, 0] * fix_flow, dtype=bool)
+    second_column_has_closed = np.array(receiver_closed[:, 1] * fix_flow, dtype=bool)
 
     # remove the link to the closed node
     receivers[first_column_has_closed, 0] = -1
     receivers[second_column_has_closed, 1] = -1
 
     # change the proportions
-    proportions[first_column_has_closed,0] = 0.
-    proportions[first_column_has_closed,1] = 1.
+    proportions[first_column_has_closed, 0] = 0.
+    proportions[first_column_has_closed, 1] = 1.
 
     proportions[second_column_has_closed, 0] = 1.
     proportions[second_column_has_closed, 1] = 0.
@@ -400,9 +430,9 @@ def flow_directions_dinf(grid,
 
     # identify the steepest link so that the steepest receiver, link, and slope
     # can be returned.
-    slope_sort = np.argsort(np.argsort(slopes_to_receivers,
-                                       axis=1),
-                            axis=1) == (num_receivers-1)
+    slope_sort = np.argsort(np.argsort(slopes_to_receivers, axis=1), axis=1) == (
+        num_receivers - 1
+    )
     steepest_slope = slopes_to_receivers[slope_sort]
     steepest_slope[drains_to_self] = 0.
 
@@ -413,25 +443,34 @@ def flow_directions_dinf(grid,
     steepest_receiver = receivers[slope_sort]
     steepest_receiver[drains_to_self] = node_id[drains_to_self]
 
-
     # Optionally, handle baselevel nodes: they are their own receivers
     if baselevel_nodes is not None:
-        receivers[baselevel_nodes,0] = node_id[baselevel_nodes]
-        receivers[baselevel_nodes,1:] = -1
+        receivers[baselevel_nodes, 0] = node_id[baselevel_nodes]
+        receivers[baselevel_nodes, 1:] = -1
         proportions[baselevel_nodes, 0] = 1
         proportions[baselevel_nodes, 1:] = 0
-        receiver_links[baselevel_nodes,:] = UNDEFINED_INDEX
+        receiver_links[baselevel_nodes, :] = UNDEFINED_INDEX
         steepest_slope[baselevel_nodes] = 0.
 
     # The sink nodes are those that are their own receivers (this will normally
     # include boundary nodes as well as interior ones; "pits" would be sink
     # nodes that are also interior nodes).
-    (sink, ) = np.where(node_id == receivers[:, 0])
+    (sink,) = np.where(node_id == receivers[:, 0])
     sink = as_id_array(sink)
 
-    return (receivers, proportions, steepest_slope, steepest_receiver, sink,
-            receiver_links, steepest_link)
+    return (
+        receivers,
+        proportions,
+        slopes_to_receivers,
+        steepest_slope,
+        steepest_receiver,
+        sink,
+        receiver_links,
+        steepest_link,
+    )
 
-if __name__ == '__main__':
+
+if __name__ == "__main__":  # pragma: no cover
     import doctest
+
     doctest.testmod()
