@@ -9,41 +9,39 @@ in that it does not consider diagonal links between nodes. For that capability,
 use FlowDirectorD8.
 """
 
-from landlab.components.flow_director.flow_director_to_many import(
-_FlowDirectorToMany)
+from landlab.components.flow_director.flow_director_to_many import _FlowDirectorToMany
 from landlab.components.flow_director import flow_direction_mfd
 from landlab import VoronoiDelaunayGrid
-from landlab import (FIXED_VALUE_BOUNDARY, FIXED_GRADIENT_BOUNDARY,
-                     BAD_INDEX_VALUE)
+from landlab import FIXED_VALUE_BOUNDARY, FIXED_GRADIENT_BOUNDARY, BAD_INDEX_VALUE
 import numpy
 
 
 class FlowDirectorMFD(_FlowDirectorToMany):
 
-    """Single-path (steepest direction) flow direction without diagonals.
+    """Multiple-path flow direction with or without out diagonals.
 
-    This components finds the steepest single-path steepest descent flow
-    directions. It is equivalent to D4 method in the special case of a raster
-    grid in that it does not consider diagonal links between nodes. For that
-    capability, use FlowDirectorD8.
+    Directs flow by the multiple flow direction method. Each node is assigned
+    multiple flow directions, toward all of the N neighboring nodes that are
+    lower than it. If none of the neighboring nodes are lower, the location is
+    identified as a pit. Flow proportions can be calculated as proportional to
+    slope or proportional to the square root of slope, which is the solution to
+    a steady kinematic wave.
 
     Specifically, it stores as ModelGrid fields:
 
     -  Node array of receivers (nodes that receive flow), or ITS OWN ID if
-       there is no receiver: *'flow__receiver_nodes'*. This array is 2D, and is
+       there is no receiver: *'flow__receiver_node'*. This array is 2D, and is
        of dimension (number of nodes x max number of receivers).
     -  Node array of flow proportions: *'flow__receiver_proportions'*. This
        array is 2D, and is of dimension (number of nodes x max number of
        receivers).
-    -  Node array of links carrying flow:  *'flow__links_to_receiver_nodes'*.
+    -  Node array of links carrying flow:  *'flow__link_to_receiver_node'*.
        This array is 2D, and is of dimension (number of nodes x max number of
        receivers).
-    -  Node array of the steepest downhill receiver. *'flow__receiver_nodes'*
-    -  Node array of steepest downhill slope from each receiver:
-       *'topographic__steepest_slope'*
-    -  Node array containing ID of steepest link that leads from each node to a
-       receiver, or BAD_INDEX_VALUE if no link:
-       *'flow__link_to_receiver_node'*
+    -  Node array of downhill slopes cooresponding to each receiver.:
+       *'topographic__steepest_slope'* This array is 2D, and is
+       of dimension (number of nodes x max number of receivers). If the slope is
+       uphill or flat, the value is assigned zero.
     -  Boolean node array of all local lows: *'flow__sink_flag'*
 
     The primary method of this class is :func:`run_one_step`.
@@ -76,7 +74,7 @@ class FlowDirectorMFD(_FlowDirectorToMany):
     directs flow to all downstream nodes. It stores the receiver information
     is a (number of nodes x maximum number or receivers) shape field at nodes.
 
-    >>> mg.at_node['flow__receiver_nodes']
+    >>> mg.at_node['flow__receiver_node']
     array([[ 0, -1, -1, -1, -1, -1, -1, -1],
            [ 1, -1, -1, -1, -1, -1, -1, -1],
            [ 2, -1, -1, -1, -1, -1, -1, -1],
@@ -87,8 +85,8 @@ class FlowDirectorMFD(_FlowDirectorToMany):
            [ 7, -1, -1, -1, -1, -1, -1, -1],
            [ 8, -1, -1, -1, -1, -1, -1, -1]])
 
-    It also stores the proportions of flow going to each receiver and the link
-    on which the flow moves in at node arrays.
+    It also stores the proportions of flow going to each receiver, the link on
+    which the flow moves in at node arrays, and the slope of each link.
 
     >>> mg.at_node['flow__receiver_proportions'] # doctest: +NORMALIZE_WHITESPACE
     array([[ 1.        ,  0.        ,  0.        ,  0.        ,  0.        ,
@@ -109,7 +107,7 @@ class FlowDirectorMFD(_FlowDirectorToMany):
              0.        ,  0.        ,  0.        ],
            [ 1.        ,  0.        ,  0.        ,  0.        ,  0.        ,
              0.        ,  0.        ,  0.        ]])
-    >>> mg.at_node['flow__links_to_receiver_nodes']
+    >>> mg.at_node['flow__link_to_receiver_node']
     array([[-1, -1, -1, -1, -1, -1, -1, -1],
            [-1, -1, -1, -1, -1, -1, -1, -1],
            [-1, -1, -1, -1, -1, -1, -1, -1],
@@ -119,19 +117,25 @@ class FlowDirectorMFD(_FlowDirectorToMany):
            [-1, -1, -1, -1, -1, -1, -1, -1],
            [-1, -1, -1, -1, -1, -1, -1, -1],
            [-1, -1, -1, -1, -1, -1, -1, -1]])
-
-    Like flow directors that only direct flow to one downstream node,
-    FlowDirectorMFD identifies and stores the steepest slope leading downhill
-    from each node, the link carrying that flow, and the receiver receiving
-    flow on that link.
-
     >>> mg.at_node['topographic__steepest_slope'] # doctest: +NORMALIZE_WHITESPACE
-    array([ 0.        ,  0.        ,  0.        ,  0.        ,  1.41421356,
-            0.        ,  0.        ,  0.        ,  0.        ])
-    >>> mg.at_node['flow__link_to_receiver_node']
-    array([-1, -1, -1, -1, 12, -1, -1, -1, -1])
-    >>> mg.at_node['flow__receiver_node']
-    array([0, 1, 2, 3, 0, 5, 6, 7, 8])
+    array([[ 0.        ,  0.        ,  0.        ,  0.        ,  0.        ,
+             0.        ,  0.        ,  0.        ],
+           [ 0.        ,  0.        ,  0.        ,  0.        ,  0.        ,
+             0.        ,  0.        ,  0.        ],
+           [ 0.        ,  0.        ,  0.        ,  0.        ,  0.        ,
+             0.        ,  0.        ,  0.        ],
+           [ 0.        ,  0.        ,  0.        ,  0.        ,  0.        ,
+             0.        ,  0.        ,  0.        ],
+           [ 0.        ,  0.        ,  0.        ,  1.        ,  0.        ,
+             0.        ,  1.41421356,  0.        ],
+           [ 0.        ,  0.        ,  0.        ,  0.        ,  0.        ,
+             0.        ,  0.        ,  0.        ],
+           [ 0.        ,  0.        ,  0.        ,  0.        ,  0.        ,
+             0.        ,  0.        ,  0.        ],
+           [ 0.        ,  0.        ,  0.        ,  0.        ,  0.        ,
+             0.        ,  0.        ,  0.        ],
+           [ 0.        ,  0.        ,  0.        ,  0.        ,  0.        ,
+             0.        ,  0.        ,  0.        ]])
 
     Finally, FlowDirectorMFD identifies sinks, or local lows.
 
@@ -193,7 +197,7 @@ class FlowDirectorMFD(_FlowDirectorToMany):
             2.5,  3.5,  4.5,  5.5,
             3. ,  4. ,  5. ])
     >>> fd.run_one_step()
-    >>> mg.at_node['flow__receiver_nodes']
+    >>> mg.at_node['flow__receiver_node']
     array([[ 0, -1, -1, -1, -1, -1],
            [ 1, -1, -1, -1, -1, -1],
            [ 2, -1, -1, -1, -1, -1],
@@ -252,7 +256,7 @@ class FlowDirectorMFD(_FlowDirectorToMany):
              0.        ],
            [ 1.        ,  0.        ,  0.        ,  0.        ,  0.        ,
              0.        ]])
-    >>> mg.at_node['flow__links_to_receiver_nodes']
+    >>> mg.at_node['flow__link_to_receiver_node']
     array([[-1, -1, -1, -1, -1, -1],
            [-1, -1, -1, -1, -1, -1],
            [-1, -1, -1, -1, -1, -1],
@@ -273,14 +277,25 @@ class FlowDirectorMFD(_FlowDirectorToMany):
            [-1, -1, -1, -1, -1, -1],
            [-1, -1, -1, -1, -1, -1]])
     >>> mg.at_node['topographic__steepest_slope'] # doctest: +NORMALIZE_WHITESPACE
-    array([ 0. ,  0. ,  0. ,  0. ,  1.5,  1.5,  0. ,  0. ,  1.5,  1.5,  1.5,
-            0. ,  0. ,  1.5,  1.5,  0. ,  0. ,  0. ,  0. ])
-    >>> mg.at_node['flow__link_to_receiver_node']
-    array([-1, -1, -1,
-           -1,  3,  5, 10,
-           -1, 12, 14, 16, 22,
-           24, 25, 27, 29,
-           -1, 36, 38])
+    array([[ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ],
+           [ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ],
+           [ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ],
+           [ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ],
+           [ 0. ,  0. ,  0. ,  1. ,  1.5,  0.5],
+           [ 0. ,  0. ,  0. ,  1. ,  1.5,  0.5],
+           [ 0. ,  0. ,  1. ,  0. ,  0. ,  0. ],
+           [ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ],
+           [ 0. ,  0. ,  0. ,  1. ,  1.5,  0.5],
+           [ 0. ,  0. ,  0. ,  1. ,  1.5,  0.5],
+           [ 0. ,  0. ,  0. ,  1. ,  1.5,  0.5],
+           [ 0. ,  1. ,  0. ,  0. ,  0. ,  0. ],
+           [ 0. ,  0. ,  0. ,  0.5,  0. ,  0. ],
+           [ 0. ,  0. ,  0.5,  1. ,  1.5,  0.5],
+           [ 0. ,  0. ,  0.5,  1. ,  1.5,  0.5],
+           [ 0. ,  1. ,  1.5,  0. ,  0. ,  0. ],
+           [ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ],
+           [ 0. ,  0. ,  0.5,  0. ,  0. ,  0. ],
+           [ 0. ,  0.5,  0. ,  0. ,  0. ,  0. ]])
     >>> mg.at_node['flow__sink_flag']
     array([1, 1, 1,
            1, 0, 0, 1,
@@ -289,9 +304,9 @@ class FlowDirectorMFD(_FlowDirectorToMany):
            1, 1, 1], dtype=int8)
     """
 
-    _name = 'FlowDirectorMFD'
+    _name = "FlowDirectorMFD"
 
-    def __init__(self, grid, surface='topographic__elevation', **kwargs):
+    def __init__(self, grid, surface="topographic__elevation", **kwargs):
         """
         Parameters
         ----------
@@ -305,16 +320,10 @@ class FlowDirectorMFD(_FlowDirectorToMany):
             'square_root_of_slope'.
         """
         # unpack kwargs:
-        try:
-            partition_method = kwargs.pop('partition_method')
-        except:
-            partition_method = 'slope'
-        try:
-            diagonals = kwargs.pop('diagonals')
-        except:
-            diagonals = False
+        partition_method = kwargs.get("partition_method", "slope")
+        diagonals = kwargs.get("diagonals", False)
 
-        self.method = 'MFD'
+        self.method = "MFD"
         super(FlowDirectorMFD, self).__init__(grid, surface)
         self._is_Voroni = isinstance(self._grid, VoronoiDelaunayGrid)
         if self._is_Voroni:
@@ -332,29 +341,41 @@ class FlowDirectorMFD(_FlowDirectorToMany):
 
         # set the number of recievers, proportions, and receiver links with the
         # right size.
-        self.receivers = grid.add_field('flow__receiver_nodes',
-                                        BAD_INDEX_VALUE*numpy.ones((self._grid.number_of_nodes,
-                                                                    self.max_receivers),
-                                                                   dtype=int),
-                                        at='node',
-                                        dtype=int,
-                                        noclobber=False)
+        self.receivers = grid.add_field(
+            "flow__receiver_node",
+            BAD_INDEX_VALUE
+            * numpy.ones((self._grid.number_of_nodes, self.max_receivers), dtype=int),
+            at="node",
+            dtype=int,
+            noclobber=False,
+        )
 
-        self.receiver_links = grid.add_field('flow__links_to_receiver_nodes',
-                                             BAD_INDEX_VALUE*numpy.ones((self._grid.number_of_nodes,
-                                                                         self.max_receivers),
-                                                                        dtype=int),
-                                        at='node',
-                                        dtype=int,
-                                        noclobber=False)
+        self.steepest_slope = grid.add_field(
+            "topographic__steepest_slope",
+            BAD_INDEX_VALUE
+            * numpy.ones((self._grid.number_of_nodes, self.max_receivers), dtype=float),
+            at="node",
+            dtype=float,
+            noclobber=False,
+        )
 
-        self.proportions = grid.add_field('flow__receiver_proportions',
-                                        BAD_INDEX_VALUE*numpy.ones((self._grid.number_of_nodes,
-                                                                    self.max_receivers),
-                                                                   dtype=float),
-                                        at='node',
-                                        dtype=int,
-                                        noclobber=False)
+        self.receiver_links = grid.add_field(
+            "flow__link_to_receiver_node",
+            BAD_INDEX_VALUE
+            * numpy.ones((self._grid.number_of_nodes, self.max_receivers), dtype=int),
+            at="node",
+            dtype=int,
+            noclobber=False,
+        )
+
+        self.proportions = grid.add_field(
+            "flow__receiver_proportions",
+            BAD_INDEX_VALUE
+            * numpy.ones((self._grid.number_of_nodes, self.max_receivers), dtype=float),
+            at="node",
+            dtype=int,
+            noclobber=False,
+        )
 
     def updated_boundary_conditions(self):
         """
@@ -396,10 +417,7 @@ class FlowDirectorMFD(_FlowDirectorToMany):
         is stored in the grid at:
         grid['node']['flow__receiver_nodes']
         """
-        # step 0. Check and update BCs
-        if self._bc_set_code != self.grid.bc_set_code:
-            self.updated_boundary_conditions()
-            self._bc_set_code = self.grid.bc_set_code
+        self._check_updated_bc()
 
         # step 1. Required inumpyuts for flow_directions_MFD
         # this is where diagonals are or are not included in
@@ -428,63 +446,72 @@ class FlowDirectorMFD(_FlowDirectorToMany):
             # calculate graidents across diagonals
             diag_grads = numpy.zeros(diag_links.shape)
             where_active_diag = dal >= diag_links.min()
-            active_diags_inds = dal[where_active_diag]-diag_links.min()
-            active_diag_grads = self.grid._calculate_gradients_at_d8_active_links(self.surface_values)
+            active_diags_inds = dal[where_active_diag] - diag_links.min()
+            active_diag_grads = self.grid._calculate_gradients_at_d8_active_links(
+                self.surface_values
+            )
             diag_grads[active_diags_inds] = active_diag_grads[where_active_diag]
 
             # calculate gradients on orthogonal links
             ortho_grads = self.grid.calc_grad_at_link(self.surface_values)
 
             # concatenate the diagonal and orthogonal grid elements
-            neighbors_at_node = numpy.hstack((self.grid.adjacent_nodes_at_node,
-                                              self.grid.diagonal_adjacent_nodes_at_node))
-            active_link_dir_at_node = numpy.hstack((self.grid.active_link_dirs_at_node,
-                                                    self.grid.active_diagonal_dirs_at_node))
-            link_slope = numpy.hstack((ortho_grads,
-                                       diag_grads))
+            neighbors_at_node = numpy.hstack(
+                (
+                    self.grid.adjacent_nodes_at_node,
+                    self.grid.diagonal_adjacent_nodes_at_node,
+                )
+            )
+            active_link_dir_at_node = numpy.hstack(
+                (
+                    self.grid.active_link_dirs_at_node,
+                    self.grid.active_diagonal_dirs_at_node,
+                )
+            )
+            link_slope = numpy.hstack((ortho_grads, diag_grads))
 
             links_at_node = self.grid.d8s_at_node
 
         # Step 2. Find and save base level nodes.
-        (baselevel_nodes, ) = numpy.where(
-            numpy.logical_or(self._grid.status_at_node == FIXED_VALUE_BOUNDARY,
-                             self._grid.status_at_node == FIXED_GRADIENT_BOUNDARY))
+        (baselevel_nodes,) = numpy.where(
+            numpy.logical_or(
+                self._grid.status_at_node == FIXED_VALUE_BOUNDARY,
+                self._grid.status_at_node == FIXED_GRADIENT_BOUNDARY,
+            )
+        )
 
         # Calculate flow directions
-        (self.receivers, self.proportions, steepest_slope,
-        steepest_receiver, sink,
-        receiver_links, steepest_link) = \
-        flow_direction_mfd.flow_directions_mfd(self.surface_values,
-                                               neighbors_at_node,
-                                               links_at_node,
-                                               active_link_dir_at_node,
-                                               link_slope,
-                                               baselevel_nodes=baselevel_nodes,
-                                               partition_method=self.partition_method)
+        (
+            self.receivers,
+            self.proportions,
+            slopes_to_receivers,
+            steepest_slope,
+            steepest_receiver,
+            sink,
+            receiver_links,
+            steepest_link,
+        ) = flow_direction_mfd.flow_directions_mfd(
+            self.surface_values,
+            neighbors_at_node,
+            links_at_node,
+            active_link_dir_at_node,
+            link_slope,
+            baselevel_nodes=baselevel_nodes,
+            partition_method=self.partition_method,
+        )
 
         # Save the four ouputs of this component.
-        self._grid['node']['flow__receiver_nodes'][:] = self.receivers
-        self._grid['node']['flow__receiver_node'][:] = steepest_receiver
-        self._grid['node']['flow__receiver_proportions'][:] = self.proportions
-        self._grid['node']['topographic__steepest_slope'][:] = steepest_slope
-        self._grid['node']['flow__link_to_receiver_node'][:] = steepest_link
-        self._grid['node']['flow__links_to_receiver_nodes'][:] = receiver_links
-        self._grid['node']['flow__sink_flag'][:] = numpy.zeros_like(steepest_link,
-                                                                    dtype=bool)
-        self._grid['node']['flow__sink_flag'][sink] = True
+        self._grid["node"]["flow__receiver_node"][:] = self.receivers
+        self._grid["node"]["flow__receiver_proportions"][:] = self.proportions
+        self._grid["node"]["topographic__steepest_slope"][:] = slopes_to_receivers
+        self._grid["node"]["flow__link_to_receiver_node"][:] = receiver_links
+        self._grid["node"]["flow__sink_flag"][:] = False
+        self._grid["node"]["flow__sink_flag"][sink] = True
 
         return (self.receivers, self.proportions)
 
-    @property
-    def nodes_receiving_flow(self):
-        """Return the node ids of the nodes receiving flow."""
-        return self._grid['node']['flow__receiver_nodes']
 
-    @property
-    def proportions_of_flow(self):
-        """Return the proportion of flow going to receivers."""
-        return self._grid['node']['flow__receiver_proportions']
-
-if __name__ == '__main__':
+if __name__ == "__main__":  # pragma: no cover
     import doctest
+
     doctest.testmod()
